from inspyre_toolbox.version import parse_version
from inspyre_toolbox.common.meta import *


__PROG__ = 'InspyreToolbox'
__VERSION__: str = parse_version()
__AUTHORS__: list[tuple[str, str]] = AUTHORS
